#include <extdll.h>
#include <meta_api.h>
#include <h_export.h>
#include "nsv_mm.h"

enginefuncs_t g_engfuncs;
globalvars_t * gpGlobals;
DLL_FUNCTIONS gFunctionTable;

enginefuncs_t meta_engfuncs;
gamedll_funcs_t * gpGamedllFuncs;
mutil_funcs_t * gpMetaUtilFuncs;
meta_globals_t * gpMetaGlobals;

static cvar_t nsv_cvars [] =
{
	{ "nsv_list", "", FCVAR_EXTDLL | FCVAR_UNLOGGED | FCVAR_SERVER, 0, NULL },
	{ "nsv_build_list", "0", FCVAR_EXTDLL, 0, NULL },
	{ NULL, NULL, 0, 0, NULL },
};

cvar_t *    nsv_list;
cvar_t *    nsv_build_list;

META_FUNCTIONS gMetaFunctionTable =
{
NULL,               // pfnGetEntityAPI()
NULL,               // pfnGetEntityAPI_Post()
GetEntityAPI2,      // pfnGetEntityAPI2()
NULL,               // pfnGetEntityAPI2_Post()
NULL,               // pfnGetNewDLLFunctions()
NULL,               // pfnGetNewDLLFunctions_Post()
GetEngineFunctions, // pfnGetEngineFunctions()
NULL,               // pfnGetEngineFunctions_Post()
};

plugin_info_t Plugin_info =
{
META_INTERFACE_VERSION,                     // interface version
"NoSeeVar",                             // plugin name
"1.00",                                      // plugin version
"02/12/2004",                               // date of creation
"BuzzKill <buzzkill@100acrebloodbath.com>", // plugin author
"http://www.100acrebloodbath.com",          // plugin URL
"NCV",                                      // plugin logtag
PT_CHANGELEVEL,                             // when loadable
PT_CHANGELEVEL,                             // when unloadable
};

C_DLLEXPORT int Meta_Query( char * ifvers, plugin_info_t ** pPlugInfo, mutil_funcs_t* pMetaUtilFuncs )
{
	// this function is the first function ever called by metamod in the plugin DLL. Its purpose
	// is for metamod to retrieve basic information about the plugin, such as its meta-interface
	// version, for ensuring compatibility with the current version of the running metamod.

	// keep track of the pointers to metamod function tables metamod gives us
	gpMetaUtilFuncs = pMetaUtilFuncs;

	* pPlugInfo = & Plugin_info;

	// check for interface version compatibility
	if (strcmp(ifvers, Plugin_info.ifvers) != 0)
	{
		int mmajor = 0, mminor = 0, pmajor = 0, pminor = 0;

		LOG_CONSOLE(PLID,   "%s: meta-interface version mismatch (metamod: %s, %s: %s)", Plugin_info.name,
					ifvers, Plugin_info.name,                                            Plugin_info.ifvers);
		LOG_MESSAGE(PLID,   "%s: meta-interface version mismatch (metamod: %s, %s: %s)", Plugin_info.name,
					ifvers, Plugin_info.name,                                            Plugin_info.ifvers);

		// if plugin has later interface version, it's incompatible (update metamod)
		sscanf(ifvers, "%d:%d", & mmajor, & mminor);
		sscanf(META_INTERFACE_VERSION, "%d:%d", & pmajor, & pminor);

		if ((pmajor>mmajor) || ((pmajor == mmajor) && (pminor>mminor)))
		{
			LOG_CONSOLE(PLID, "metamod version is too old for this plugin; update metamod");

			LOG_ERROR(PLID, "metamod version is too old for this plugin; update metamod");
			return (FALSE);
		}

		// if plugin has older major interface version, it's incompatible (update plugin)
		else if (pmajor<mmajor)
		{
			LOG_CONSOLE(PLID,
						"metamod version is incompatible with this plugin; please find a newer version of this plugin");

			LOG_ERROR(PLID,
					  "metamod version is incompatible with this plugin; please find a newer version of this plugin");
			return (FALSE);
		}
	}

	return (TRUE); // tell metamod this plugin looks safe
}

C_DLLEXPORT int Meta_Attach( PLUG_LOADTIME now, META_FUNCTIONS* pFunctionTable, meta_globals_t* pMGlobals,
							 gamedll_funcs_t* pGamedllFuncs )
{
	// this function is called when metamod attempts to load the plugin. Since it's the place
	// where we can tell if the plugin will be allowed to run or not, we wait until here to make
	// our initialization stuff, like registering CVARs and dedicated server commands.

	// are we allowed to load this plugin now ?
	if (now>Plugin_info.loadable)
	{
		LOG_CONSOLE(PLID, "%s: plugin NOT attaching (can't load plugin right now)", Plugin_info.name);

		LOG_ERROR(PLID, "%s: plugin NOT attaching (can't load plugin right now)", Plugin_info.name);
		return (FALSE); // returning FALSE prevents metamod from attaching this plugin
	}

	// keep track of the pointers to engine function tables metamod gives us
	gpMetaGlobals = pMGlobals;
	memcpy(pFunctionTable, & gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs = pGamedllFuncs;

	// print a message to notify about plugin attaching
	LOG_CONSOLE(PLID, "%s: plugin attaching", Plugin_info.name);
	LOG_MESSAGE(PLID, "%s: plugin attaching", Plugin_info.name);


	// process the stealth list BEFORE we register our own so we can block our own if we want.
	

	for(int i = 0; nsv_cvars[i].name; i++) CVAR_REGISTER( &nsv_cvars[i] );
	nsv_list = CVAR_GET_POINTER( "nsv_list" );
	nsv_build_list = CVAR_GET_POINTER( "nsv_build_list" );

	process_cfg();

	CVAR_DIRSET_STRING(nsv_list, cvar_list);
	

	return (TRUE); // returning TRUE enables metamod to attach this plugin
}

C_DLLEXPORT int Meta_Detach( PLUG_LOADTIME now, PL_UNLOAD_REASON reason )
{
	// this function is called when metamod unloads the plugin. A basic check is made in order
	// to prevent unloading the plugin if its processing should not be interrupted.

	// is metamod allowed to unload the plugin ?
	if ((now>Plugin_info.unloadable) && (reason != PNL_CMD_FORCED))
	{
		LOG_CONSOLE(PLID, "%s: plugin NOT detaching (can't unload plugin right now)", Plugin_info.name);

		LOG_ERROR(PLID, "%s: plugin NOT detaching (can't unload plugin right now)", Plugin_info.name);
		return (FALSE); // returning FALSE prevents metamod from unloading this plugin
	}

	return (TRUE);      // returning TRUE enables metamod to unload this plugin
}

C_DLLEXPORT int GetEntityAPI2( DLL_FUNCTIONS* pFunctionTable, int* interfaceVersion )
{
	// this is one of the initialization functions hooked by metamod in the gamedll API

	// keep track of pointers to the functions we use in the gamedll API

	// copy the whole table for metamod to know which functions we are using here
	memcpy(pFunctionTable, & gFunctionTable, sizeof(DLL_FUNCTIONS));

	return (TRUE); // alright
}

C_DLLEXPORT int GetEngineFunctions( enginefuncs_t* pengfuncsFromEngine, int* interfaceVersion )
{
	// this is one of the initialization functions hooked by metamod in the engine API

	// keep track of pointers to the functions we use in the engine API

	// copy the whole table for metamod to know which functions we are using here
	memcpy(pengfuncsFromEngine, & meta_engfuncs, sizeof(enginefuncs_t));

	return (TRUE); // alright
}

// END of Metamod stuff

#ifdef _WIN32

BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved ) {
// required Win32 DLL entry point. Good place for freeing stuff upon DLL unloading (in case
// fdwReason would be DLL_PROCESS_DETACH) if needed. The same for Linux would be implemented
// in an optional _fini() function.

	return (TRUE); // nothing to do here
}

#endif

void WINAPI GiveFnptrsToDll( enginefuncs_t* pengfuncsFromEngine, globalvars_t* pGlobals )
{
	// this function is called by the engine to tell the MOD DLL (hence the plugins too) the
	// address of its function table and its server global variables structure.

	// keep track of the engine functions and the server globals for our own use
	memcpy(& g_engfuncs, pengfuncsFromEngine, sizeof(enginefuncs_t));

	gpGlobals = pGlobals;
}


